@@ -1,18 +1,8 @@
 /*
- * Copyright (C) 2012 - David Goulet <dgoulet@efficios.com>
+ * Copyright (C) 2012 David Goulet <dgoulet@efficios.com>
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License, version 2 only, as
- * published by the Free Software Foundation.
+ * SPDX-License-Identifier: GPL-2.0-only
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 51
- * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef _CONSUMER_H
@@ -152,6 +142,9 @@ struct consumer_output {
 	uint32_t relay_major_version;
 	uint32_t relay_minor_version;
 
+	/* True if relayd supports the clear feature. */
+	bool relay_allows_clear;
+
 	/*
 	 * Subdirectory path name used for both local and network
 	 * consumer ("kernel", "ust", or empty).
@@ -240,7 +233,6 @@ void consumer_init_ask_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 		unsigned int switch_timer_interval,
 		unsigned int read_timer_interval,
 		unsigned int live_timer_interval,
-		bool is_in_live_session,
 		unsigned int monitor_timer_interval,
 		int output,
 		int type,
@@ -283,7 +275,6 @@ void consumer_init_add_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 		uint64_t tracefile_count,
 		unsigned int monitor,
 		unsigned int live_timer_interval,
-		bool is_in_live_session,
 		unsigned int monitor_timer_interval,
 		struct lttng_trace_chunk *trace_chunk);
 int consumer_is_data_pending(uint64_t session_id,
@@ -317,7 +308,8 @@ int consumer_init(struct consumer_socket *socket,
 
 int consumer_create_trace_chunk(struct consumer_socket *socket,
 		uint64_t relayd_id, uint64_t session_id,
-		struct lttng_trace_chunk *chunk);
+		struct lttng_trace_chunk *chunk,
+		const char *domain_subdir);
 int consumer_close_trace_chunk(struct consumer_socket *socket,
 		uint64_t relayd_id, uint64_t session_id,
 		struct lttng_trace_chunk *chunk,
@@ -328,6 +320,9 @@ int consumer_trace_chunk_exists(struct consumer_socket *socket,
 		enum consumer_trace_chunk_exists_status *result);
 
 char *setup_channel_trace_path(struct consumer_output *consumer,
-		const char *session_path);
+		const char *session_path, size_t *consumer_path_offset);
+
+/* Clear command */
+int consumer_clear_channel(struct consumer_socket *socket, uint64_t key);
 
 #endif /* _CONSUMER_H */
