@@ -1,20 +1,10 @@
 /*
- * Copyright (C) 2011 - Julien Desfossez <julien.desfossez@polymtl.ca>
- *                      Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
- * Copyright (C) 2017 - Jérémie Galarneau <jeremie.galarneau@efficios.com>
+ * Copyright (C) 2011 Julien Desfossez <julien.desfossez@polymtl.ca>
+ * Copyright (C) 2011 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+ * Copyright (C) 2017 Jérémie Galarneau <jeremie.galarneau@efficios.com>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License, version 2 only,
- * as published by the Free Software Foundation.
+ * SPDX-License-Identifier: GPL-2.0-only
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #define _LGPL_SOURCE
@@ -33,7 +23,6 @@
 #include <urcu/list.h>
 #include <signal.h>
 #include <stdbool.h>
-#include <stdint.h>
 
 #include <bin/lttng-consumerd/health-consumerd.h>
 #include <common/common.h>
@@ -46,8 +35,6 @@
 #include <common/consumer/consumer-timer.h>
 #include <common/utils.h>
 #include <common/index/index.h>
-#include <common/consumer/consumer.h>
-#include <common/optional.h>
 
 #include "ust-consumer.h"
 
@@ -88,11 +75,6 @@ static void destroy_channel(struct lttng_consumer_channel *channel)
 		lttng_ustconsumer_del_channel(channel);
 		lttng_ustconsumer_free_channel(channel);
 	}
-
-	if (channel->trace_chunk) {
-		lttng_trace_chunk_put(channel->trace_chunk);
-	}
-
 	free(channel);
 }
 
@@ -128,6 +110,26 @@ error:
 	return ret;
 }
 
+/*
+ * Allocate and return a consumer channel object.
+ */
+static struct lttng_consumer_channel *allocate_channel(uint64_t session_id,
+		const uint64_t *chunk_id, const char *pathname, const char *name,
+		uint64_t relayd_id, uint64_t key, enum lttng_event_output output,
+		uint64_t tracefile_size, uint64_t tracefile_count,
+		uint64_t session_id_per_pid, unsigned int monitor,
+		unsigned int live_timer_interval,
+		const char *root_shm_path, const char *shm_path)
+{
+	assert(pathname);
+	assert(name);
+
+	return consumer_allocate_channel(key, session_id, chunk_id, pathname,
+			name, relayd_id, output, tracefile_size,
+			tracefile_count, session_id_per_pid, monitor,
+			live_timer_interval, root_shm_path, shm_path);
+}
+
 /*
  * Allocate and return a consumer stream object. If _alloc_ret is not NULL, the
  * error value if applicable is set in it else it is kept untouched.
@@ -144,9 +146,7 @@ static struct lttng_consumer_stream *allocate_stream(int cpu, int key,
 	assert(channel);
 	assert(ctx);
 
-	stream = consumer_stream_create(
-			channel,
-			channel->key,
+	stream = consumer_allocate_stream(channel->key,
 			key,
 			channel->name,
 			channel->relayd_id,
@@ -175,6 +175,7 @@ static struct lttng_consumer_stream *allocate_stream(int cpu, int key,
 	}
 
 	consumer_stream_update_channel_attributes(stream, channel);
+	stream->chan = channel;
 
 error:
 	if (_alloc_ret) {
@@ -1057,7 +1058,7 @@ static int snapshot_metadata(struct lttng_consumer_channel *metadata_channel,
 	do {
 		health_code_update();
 
-		ret = lttng_consumer_read_subbuffer(metadata_stream, ctx, true);
+		ret = lttng_consumer_read_subbuffer(metadata_stream, ctx);
 		if (ret < 0) {
 			goto error_stream;
 		}
@@ -1077,35 +1078,6 @@ error:
 	return ret;
 }
 
-static
-int get_current_subbuf_addr(struct lttng_consumer_stream *stream,
-		const char **addr)
-{
-	int ret;
-	unsigned long mmap_offset;
-	const char *mmap_base;
-
-	mmap_base = ustctl_get_mmap_base(stream->ustream);
-	if (!mmap_base) {
-		ERR("Failed to get mmap base for stream `%s`",
-				stream->name);
-		ret = -EPERM;
-		goto error;
-	}
-
-	ret = ustctl_get_mmap_read_offset(stream->ustream, &mmap_offset);
-	if (ret != 0) {
-		ERR("Failed to get mmap offset for stream `%s`", stream->name);
-		ret = -EINVAL;
-		goto error;
-	}
-
-	*addr = mmap_base + mmap_offset;
-error:
-	return ret;
-
-}
-
 /*
  * Take a snapshot of all the stream of a channel.
  * RCU read-side lock and the channel lock must be held by the caller.
@@ -1208,8 +1180,6 @@ static int snapshot_channel(struct lttng_consumer_channel *channel,
 		while ((long) (consumed_pos - produced_pos) < 0) {
 			ssize_t read_len;
 			unsigned long len, padded_len;
-			const char *subbuf_addr;
-			struct lttng_buffer_view subbuf_view;
 
 			health_code_update();
 
@@ -1239,15 +1209,8 @@ static int snapshot_channel(struct lttng_consumer_channel *channel,
 				goto error_put_subbuf;
 			}
 
-			ret = get_current_subbuf_addr(stream, &subbuf_addr);
-			if (ret) {
-				goto error_put_subbuf;
-			}
-
-			subbuf_view = lttng_buffer_view_init(
-					subbuf_addr, 0, padded_len);
-			read_len = lttng_consumer_on_read_subbuffer_mmap(
-					stream, &subbuf_view, padded_len - len);
+			read_len = lttng_consumer_on_read_subbuffer_mmap(ctx, stream, len,
+					padded_len - len, NULL);
 			if (use_relayd) {
 				if (read_len != len) {
 					ret = -EPERM;
@@ -1476,21 +1439,19 @@ int lttng_ustconsumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 		};
 
 		/* Create a plain object and reserve a channel key. */
-		channel = consumer_allocate_channel(
-				msg.u.ask_channel.key,
-				msg.u.ask_channel.session_id,
+		channel = allocate_channel(msg.u.ask_channel.session_id,
 				msg.u.ask_channel.chunk_id.is_set ?
 						&chunk_id : NULL,
 				msg.u.ask_channel.pathname,
 				msg.u.ask_channel.name,
 				msg.u.ask_channel.relayd_id,
+				msg.u.ask_channel.key,
 				(enum lttng_event_output) msg.u.ask_channel.output,
 				msg.u.ask_channel.tracefile_size,
 				msg.u.ask_channel.tracefile_count,
 				msg.u.ask_channel.session_id_per_pid,
 				msg.u.ask_channel.monitor,
 				msg.u.ask_channel.live_timer_interval,
-				msg.u.ask_channel.is_live,
 				msg.u.ask_channel.root_shm_path,
 				msg.u.ask_channel.shm_path);
 		if (!channel) {
@@ -2042,6 +2003,31 @@ error_push_metadata_fatal:
 end_rotate_channel_nosignal:
 		goto end_nosignal;
 	}
+	case LTTNG_CONSUMER_CLEAR_CHANNEL:
+	{
+		struct lttng_consumer_channel *channel;
+		uint64_t key = msg.u.clear_channel.key;
+
+		channel = consumer_find_channel(key);
+		if (!channel) {
+			DBG("Channel %" PRIu64 " not found", key);
+			ret_code = LTTCOMM_CONSUMERD_CHAN_NOT_FOUND;
+		} else {
+			ret = lttng_consumer_clear_channel(channel);
+			if (ret) {
+				ERR("Clear channel failed key %" PRIu64, key);
+				ret_code = ret;
+			}
+
+			health_code_update();
+		}
+		ret = consumer_send_status_msg(sock, ret_code);
+		if (ret < 0) {
+			/* Somehow, the session daemon is not responding anymore. */
+			goto end_nosignal;
+		}
+		break;
+	}
 	case LTTNG_CONSUMER_INIT:
 	{
 		ret_code = lttng_consumer_init_command(ctx,
@@ -2068,8 +2054,7 @@ end_rotate_channel_nosignal:
 				*msg.u.create_trace_chunk.override_name ?
 					msg.u.create_trace_chunk.override_name :
 					NULL;
-		LTTNG_OPTIONAL(struct lttng_directory_handle) chunk_directory_handle =
-				LTTNG_OPTIONAL_INIT;
+		struct lttng_directory_handle *chunk_directory_handle = NULL;
 
 		/*
 		 * The session daemon will only provide a chunk directory file
@@ -2086,25 +2071,26 @@ end_rotate_channel_nosignal:
 				goto end_nosignal;
 			}
 
+			/*
+			 * Receive trace chunk domain dirfd.
+			 */
 			ret = lttcomm_recv_fds_unix_sock(sock, &chunk_dirfd, 1);
 			if (ret != sizeof(chunk_dirfd)) {
-				ERR("Failed to receive trace chunk directory file descriptor");
+				ERR("Failed to receive trace chunk domain directory file descriptor");
 				goto error_fatal;
 			}
 
-			DBG("Received trace chunk directory fd (%d)",
+			DBG("Received trace chunk domain directory fd (%d)",
 					chunk_dirfd);
-			ret = lttng_directory_handle_init_from_dirfd(
-					&chunk_directory_handle.value,
+			chunk_directory_handle = lttng_directory_handle_create_from_dirfd(
 					chunk_dirfd);
-			if (ret) {
-				ERR("Failed to initialize chunk directory handle from directory file descriptor");
+			if (!chunk_directory_handle) {
+				ERR("Failed to initialize chunk domain directory handle from directory file descriptor");
 				if (close(chunk_dirfd)) {
 					PERROR("Failed to close chunk directory file descriptor");
 				}
 				goto error_fatal;
 			}
-			chunk_directory_handle.is_set = true;
 		}
 
 		ret_code = lttng_consumer_create_trace_chunk(
@@ -2117,14 +2103,8 @@ end_rotate_channel_nosignal:
 				msg.u.create_trace_chunk.credentials.is_set ?
 						&credentials :
 						NULL,
-				chunk_directory_handle.is_set ?
-						&chunk_directory_handle.value :
-						NULL);
-
-		if (chunk_directory_handle.is_set) {
-			lttng_directory_handle_fini(
-					&chunk_directory_handle.value);
-		}
+				chunk_directory_handle);
+		lttng_directory_handle_put(chunk_directory_handle);
 		goto end_msg_sessiond;
 	}
 	case LTTNG_CONSUMER_CLOSE_TRACE_CHUNK:
@@ -2225,6 +2205,31 @@ end:
 	return ret;
 }
 
+/*
+ * Wrapper over the mmap() read offset from ust-ctl library. Since this can be
+ * compiled out, we isolate it in this library.
+ */
+int lttng_ustctl_get_mmap_read_offset(struct lttng_consumer_stream *stream,
+		unsigned long *off)
+{
+	assert(stream);
+	assert(stream->ustream);
+
+	return ustctl_get_mmap_read_offset(stream->ustream, off);
+}
+
+/*
+ * Wrapper over the mmap() read offset from ust-ctl library. Since this can be
+ * compiled out, we isolate it in this library.
+ */
+void *lttng_ustctl_get_mmap_base(struct lttng_consumer_stream *stream)
+{
+	assert(stream);
+	assert(stream->ustream);
+
+	return ustctl_get_mmap_base(stream->ustream);
+}
+
 void lttng_ustctl_flush_buffer(struct lttng_consumer_stream *stream,
 		int producer_active)
 {
@@ -2300,6 +2305,14 @@ void lttng_ustconsumer_flush_buffer(struct lttng_consumer_stream *stream,
 	ustctl_flush_buffer(stream->ustream, producer);
 }
 
+void lttng_ustconsumer_clear_buffer(struct lttng_consumer_stream *stream)
+{
+	assert(stream);
+	assert(stream->ustream);
+
+	ustctl_clear_buffer(stream->ustream);
+}
+
 int lttng_ustconsumer_get_current_timestamp(
 		struct lttng_consumer_stream *stream, uint64_t *ts)
 {
@@ -2417,20 +2430,120 @@ int lttng_ustconsumer_close_wakeup_fd(struct lttng_consumer_stream *stream)
 	return ustctl_stream_close_wakeup_fd(stream->ustream);
 }
 
-static
-void metadata_stream_reset_cache_consumed_position(
-		struct lttng_consumer_stream *stream)
+/*
+ * Populate index values of a UST stream. Values are set in big endian order.
+ *
+ * Return 0 on success or else a negative value.
+ */
+static int get_index_values(struct ctf_packet_index *index,
+		struct ustctl_consumer_stream *ustream)
 {
-	DBG("Reset metadata cache of session %" PRIu64,
-			stream->chan->session_id);
+	int ret;
+	uint64_t packet_size, content_size, timestamp_begin, timestamp_end,
+			events_discarded, stream_id, stream_instance_id,
+			packet_seq_num;
+
+	ret = ustctl_get_timestamp_begin(ustream, &timestamp_begin);
+	if (ret < 0) {
+		PERROR("ustctl_get_timestamp_begin");
+		goto error;
+	}
+
+	ret = ustctl_get_timestamp_end(ustream, &timestamp_end);
+	if (ret < 0) {
+		PERROR("ustctl_get_timestamp_end");
+		goto error;
+	}
+
+	ret = ustctl_get_events_discarded(ustream, &events_discarded);
+	if (ret < 0) {
+		PERROR("ustctl_get_events_discarded");
+		goto error;
+	}
+
+	ret = ustctl_get_content_size(ustream, &content_size);
+	if (ret < 0) {
+		PERROR("ustctl_get_content_size");
+		goto error;
+	}
+
+	ret = ustctl_get_packet_size(ustream, &packet_size);
+	if (ret < 0) {
+		PERROR("ustctl_get_packet_size");
+		goto error;
+	}
+
+	ret = ustctl_get_stream_id(ustream, &stream_id);
+	if (ret < 0) {
+		PERROR("ustctl_get_stream_id");
+		goto error;
+	}
+
+	ret = ustctl_get_instance_id(ustream, &stream_instance_id);
+	if (ret < 0) {
+		PERROR("ustctl_get_instance_id");
+		goto error;
+	}
+
+	ret = ustctl_get_sequence_number(ustream, &packet_seq_num);
+	if (ret < 0) {
+		PERROR("ustctl_get_sequence_number");
+		goto error;
+	}
+
+	*index = (typeof(*index)) {
+		.offset = index->offset,
+		.packet_size = htobe64(packet_size),
+		.content_size = htobe64(content_size),
+		.timestamp_begin = htobe64(timestamp_begin),
+		.timestamp_end = htobe64(timestamp_end),
+		.events_discarded = htobe64(events_discarded),
+		.stream_id = htobe64(stream_id),
+		.stream_instance_id = htobe64(stream_instance_id),
+		.packet_seq_num = htobe64(packet_seq_num),
+	};
+
+error:
+	return ret;
+}
+
+static
+void metadata_stream_reset_cache(struct lttng_consumer_stream *stream,
+		struct consumer_metadata_cache *cache)
+{
+	DBG("Metadata stream update to version %" PRIu64,
+			cache->version);
 	stream->ust_metadata_pushed = 0;
+	stream->metadata_version = cache->version;
+	stream->reset_metadata_flag = 1;
+}
+
+/*
+ * Check if the version of the metadata stream and metadata cache match.
+ * If the cache got updated, reset the metadata stream.
+ * The stream lock and metadata cache lock MUST be held.
+ * Return 0 on success, a negative value on error.
+ */
+static
+int metadata_stream_check_version(struct lttng_consumer_stream *stream)
+{
+	int ret = 0;
+	struct consumer_metadata_cache *cache = stream->chan->metadata_cache;
+
+	if (cache->version == stream->metadata_version) {
+		goto end;
+	}
+	metadata_stream_reset_cache(stream, cache);
+
+end:
+	return ret;
 }
 
 /*
  * Write up to one packet from the metadata cache to the channel.
  *
- * Returns the number of bytes pushed from the cache into the ring buffer, or a
- * negative value on error.
+ * Returns the number of bytes pushed in the cache, or a negative value
+ * on error.
  */
 static
 int commit_one_metadata_packet(struct lttng_consumer_stream *stream)
@@ -2439,41 +2552,14 @@ int commit_one_metadata_packet(struct lttng_consumer_stream *stream)
 	int ret;
 
 	pthread_mutex_lock(&stream->chan->metadata_cache->lock);
-	if (stream->chan->metadata_cache->max_offset ==
-	    stream->ust_metadata_pushed) {
-		/*
-		 * In the context of a user space metadata channel, a
-		 * change in version can be detected in two ways:
-		 *   1) During the pre-consume of the `read_subbuffer` loop,
-		 *   2) When populating the metadata ring buffer (i.e. here).
-		 *
-		 * This function is invoked when there is no metadata
-		 * available in the ring-buffer. If all data was consumed
-		 * up to the size of the metadata cache, there is no metadata
-		 * to insert in the ring-buffer.
-		 *
-		 * However, the metadata version could still have changed (a
-		 * regeneration without any new data will yield the same cache
-		 * size).
-		 *
-		 * The cache's version is checked for a version change and the
-		 * consumed position is reset if one occurred.
-		 *
-		 * This check is only necessary for the user space domain as
-		 * it has to manage the cache explicitly. If this reset was not
-		 * performed, no metadata would be consumed (and no reset would
-		 * occur as part of the pre-consume) until the metadata size
-		 * exceeded the cache size.
-		 */
-		if (stream->metadata_version !=
-				stream->chan->metadata_cache->version) {
-			metadata_stream_reset_cache_consumed_position(stream);
-			consumer_stream_metadata_set_version(stream,
-					stream->chan->metadata_cache->version);
-		} else {
-			ret = 0;
-			goto end;
-		}
+	ret = metadata_stream_check_version(stream);
+	if (ret < 0) {
+		goto end;
+	}
+	if (stream->chan->metadata_cache->max_offset
+			== stream->ust_metadata_pushed) {
+		ret = 0;
+		goto end;
 	}
 
 	write_len = ustctl_write_one_packet_to_channel(stream->chan->uchan,
@@ -2483,7 +2569,7 @@ int commit_one_metadata_packet(struct lttng_consumer_stream *stream)
 	assert(write_len != 0);
 	if (write_len < 0) {
 		ERR("Writing one metadata packet");
-		ret = write_len;
+		ret = -1;
 		goto end;
 	}
 	stream->ust_metadata_pushed += write_len;
@@ -2514,13 +2600,15 @@ end:
  * awaiting on metadata to be pushed out.
  *
  * The RCU read side lock must be held by the caller.
+ *
+ * Return 0 if new metadatda is available, EAGAIN if the metadata stream
+ * is empty or a negative value on error.
  */
-enum sync_metadata_status lttng_ustconsumer_sync_metadata(
-		struct lttng_consumer_local_data *ctx,
+int lttng_ustconsumer_sync_metadata(struct lttng_consumer_local_data *ctx,
 		struct lttng_consumer_stream *metadata_stream)
 {
 	int ret;
-	enum sync_metadata_status status;
+	int retry = 0;
 	struct lttng_consumer_channel *metadata_channel;
 
 	assert(ctx);
@@ -2535,7 +2623,6 @@ enum sync_metadata_status lttng_ustconsumer_sync_metadata(
 	ret = lttng_ustconsumer_request_metadata(ctx, metadata_channel, 0, 0);
 	pthread_mutex_lock(&metadata_stream->lock);
 	if (ret < 0) {
-		status = SYNC_METADATA_STATUS_ERROR;
 		goto end;
 	}
 
@@ -2553,30 +2640,38 @@ enum sync_metadata_status lttng_ustconsumer_sync_metadata(
 	if (consumer_stream_is_deleted(metadata_stream)) {
 		DBG("Metadata stream %" PRIu64 " was deleted during the metadata synchronization",
 				metadata_stream->key);
-		status = SYNC_METADATA_STATUS_NO_DATA;
+		ret = 0;
 		goto end;
 	}
 
 	ret = commit_one_metadata_packet(metadata_stream);
-	if (ret < 0) {
-		status = SYNC_METADATA_STATUS_ERROR;
+	if (ret <= 0) {
 		goto end;
 	} else if (ret > 0) {
-		status = SYNC_METADATA_STATUS_NEW_DATA;
-	} else /* ret == 0 */ {
-		status = SYNC_METADATA_STATUS_NO_DATA;
-		goto end;
+		retry = 1;
 	}
 
 	ret = ustctl_snapshot(metadata_stream->ustream);
 	if (ret < 0) {
-		ERR("Failed to take a snapshot of the metadata ring-buffer positions, ret = %d", ret);
-		status = SYNC_METADATA_STATUS_ERROR;
+		if (errno != EAGAIN) {
+			ERR("Sync metadata, taking UST snapshot");
+			goto end;
+		}
+		DBG("No new metadata when syncing them.");
+		/* No new metadata, exit. */
+		ret = ENODATA;
 		goto end;
 	}
 
+	/*
+	 * After this flush, we still need to extract metadata.
+	 */
+	if (retry) {
+		ret = EAGAIN;
+	}
+
 end:
-	return status;
+	return ret;
 }
 
 /*
@@ -2630,20 +2725,96 @@ end:
 	return ret;
 }
 
-static int consumer_stream_ust_on_wake_up(struct lttng_consumer_stream *stream)
+static
+int update_stream_stats(struct lttng_consumer_stream *stream)
 {
-	int ret = 0;
+	int ret;
+	uint64_t seq, discarded;
+
+	ret = ustctl_get_sequence_number(stream->ustream, &seq);
+	if (ret < 0) {
+		PERROR("ustctl_get_sequence_number");
+		goto end;
+	}
+	/*
+	 * Start the sequence when we extract the first packet in case we don't
+	 * start at 0 (for example if a consumer is not connected to the
+	 * session immediately after the beginning).
+	 */
+	if (stream->last_sequence_number == -1ULL) {
+		stream->last_sequence_number = seq;
+	} else if (seq > stream->last_sequence_number) {
+		stream->chan->lost_packets += seq -
+				stream->last_sequence_number - 1;
+	} else {
+		/* seq <= last_sequence_number */
+		ERR("Sequence number inconsistent : prev = %" PRIu64
+				", current = %" PRIu64,
+				stream->last_sequence_number, seq);
+		ret = -1;
+		goto end;
+	}
+	stream->last_sequence_number = seq;
+
+	ret = ustctl_get_events_discarded(stream->ustream, &discarded);
+	if (ret < 0) {
+		PERROR("kernctl_get_events_discarded");
+		goto end;
+	}
+	if (discarded < stream->last_discarded_events) {
+		/*
+		 * Overflow has occurred. We assume only one wrap-around
+		 * has occurred.
+		 */
+		stream->chan->discarded_events +=
+				(1ULL << (CAA_BITS_PER_LONG - 1)) -
+				stream->last_discarded_events + discarded;
+	} else {
+		stream->chan->discarded_events += discarded -
+				stream->last_discarded_events;
+	}
+	stream->last_discarded_events = discarded;
+	ret = 0;
+
+end:
+	return ret;
+}
+
+/*
+ * Read subbuffer from the given stream.
+ *
+ * Stream and channel locks MUST be acquired by the caller.
+ *
+ * Return 0 on success else a negative value.
+ */
+int lttng_ustconsumer_read_subbuffer(struct lttng_consumer_stream *stream,
+		struct lttng_consumer_local_data *ctx)
+{
+	unsigned long len, subbuf_size, padding;
+	int err, write_index = 1, rotation_ret;
+	long ret = 0;
+	struct ustctl_consumer_stream *ustream;
+	struct ctf_packet_index index;
+
+	assert(stream);
+	assert(stream->ustream);
+	assert(ctx);
+
+	DBG("In UST read_subbuffer (wait_fd: %d, name: %s)", stream->wait_fd,
+			stream->name);
+
+	/* Ease our life for what's next. */
+	ustream = stream->ustream;
 
 	/*
-	 * We can consume the 1 byte written into the wait_fd by
-	 * UST. Don't trigger error if we cannot read this one byte
-	 * (read returns 0), or if the error is EAGAIN or EWOULDBLOCK.
+	 * We can consume the 1 byte written into the wait_fd by UST. Don't trigger
+	 * error if we cannot read this one byte (read returns 0), or if the error
+	 * is EAGAIN or EWOULDBLOCK.
 	 *
-	 * This is only done when the stream is monitored by a thread,
-	 * before the flush is done after a hangup and if the stream
-	 * is not flagged with data since there might be nothing to
-	 * consume in the wait fd but still have data available
-	 * flagged by the consumer wake up pipe.
+	 * This is only done when the stream is monitored by a thread, before the
+	 * flush is done after a hangup and if the stream is not flagged with data
+	 * since there might be nothing to consume in the wait fd but still have
+	 * data available flagged by the consumer wake up pipe.
 	 */
 	if (stream->monitor && !stream->hangup_flush_done && !stream->has_data) {
 		char dummy;
@@ -2652,310 +2823,178 @@ static int consumer_stream_ust_on_wake_up(struct lttng_consumer_stream *stream)
 		readlen = lttng_read(stream->wait_fd, &dummy, 1);
 		if (readlen < 0 && errno != EAGAIN && errno != EWOULDBLOCK) {
 			ret = readlen;
+			goto error;
 		}
 	}
 
-	return ret;
-}
-
-static int extract_common_subbuffer_info(struct lttng_consumer_stream *stream,
-		struct stream_subbuffer *subbuf)
-{
-	int ret;
-
-	ret = ustctl_get_subbuf_size(
-			stream->ustream, &subbuf->info.data.subbuf_size);
-	if (ret) {
-		goto end;
-	}
-
-	ret = ustctl_get_padded_subbuf_size(
-			stream->ustream, &subbuf->info.data.padded_subbuf_size);
-	if (ret) {
-		goto end;
-	}
-
-end:
-	return ret;
-}
-
-static int extract_metadata_subbuffer_info(struct lttng_consumer_stream *stream,
-		struct stream_subbuffer *subbuf)
-{
-	int ret;
-
-	ret = extract_common_subbuffer_info(stream, subbuf);
-	if (ret) {
-		goto end;
-	}
-
-	subbuf->info.metadata.version = stream->metadata_version;
-
-end:
-	return ret;
-}
-
-static int extract_data_subbuffer_info(struct lttng_consumer_stream *stream,
-		struct stream_subbuffer *subbuf)
-{
-	int ret;
-
-	ret = extract_common_subbuffer_info(stream, subbuf);
-	if (ret) {
-		goto end;
-	}
-
-	ret = ustctl_get_packet_size(
-			stream->ustream, &subbuf->info.data.packet_size);
-	if (ret < 0) {
-		PERROR("Failed to get sub-buffer packet size");
-		goto end;
-	}
-
-	ret = ustctl_get_content_size(
-			stream->ustream, &subbuf->info.data.content_size);
-	if (ret < 0) {
-		PERROR("Failed to get sub-buffer content size");
-		goto end;
-	}
-
-	ret = ustctl_get_timestamp_begin(
-			stream->ustream, &subbuf->info.data.timestamp_begin);
-	if (ret < 0) {
-		PERROR("Failed to get sub-buffer begin timestamp");
-		goto end;
-	}
-
-	ret = ustctl_get_timestamp_end(
-			stream->ustream, &subbuf->info.data.timestamp_end);
-	if (ret < 0) {
-		PERROR("Failed to get sub-buffer end timestamp");
-		goto end;
-	}
-
-	ret = ustctl_get_events_discarded(
-			stream->ustream, &subbuf->info.data.events_discarded);
-	if (ret) {
-		PERROR("Failed to get sub-buffer events discarded count");
-		goto end;
-	}
-
-	ret = ustctl_get_sequence_number(stream->ustream,
-			&subbuf->info.data.sequence_number.value);
-	if (ret) {
-		/* May not be supported by older LTTng-modules. */
-		if (ret != -ENOTTY) {
-			PERROR("Failed to get sub-buffer sequence number");
-			goto end;
-		}
-	} else {
-		subbuf->info.data.sequence_number.is_set = true;
-	}
-
-	ret = ustctl_get_stream_id(
-			stream->ustream, &subbuf->info.data.stream_id);
-	if (ret < 0) {
-		PERROR("Failed to get stream id");
-		goto end;
-	}
-
-	ret = ustctl_get_instance_id(stream->ustream,
-			&subbuf->info.data.stream_instance_id.value);
-	if (ret) {
-		/* May not be supported by older LTTng-modules. */
-		if (ret != -ENOTTY) {
-			PERROR("Failed to get stream instance id");
-			goto end;
-		}
-	} else {
-		subbuf->info.data.stream_instance_id.is_set = true;
-	}
-end:
-	return ret;
-}
-
-static int get_next_subbuffer_common(struct lttng_consumer_stream *stream,
-		struct stream_subbuffer *subbuffer)
-{
-	int ret;
-	const char *addr;
-
-	ret = stream->read_subbuffer_ops.extract_subbuffer_info(
-			stream, subbuffer);
-	if (ret) {
-		goto end;
-	}
-
-	ret = get_current_subbuf_addr(stream, &addr);
-	if (ret) {
-		goto end;
-	}
-
-	subbuffer->buffer.buffer = lttng_buffer_view_init(
-			addr, 0, subbuffer->info.data.padded_subbuf_size);
-	assert(subbuffer->buffer.buffer.data != NULL);
-end:
-	return ret;
-}
-
-static int get_next_subbuffer(struct lttng_consumer_stream *stream,
-		struct stream_subbuffer *subbuffer)
-{
-	int ret;
-
-	ret = ustctl_get_next_subbuf(stream->ustream);
-	if (ret) {
-		goto end;
-	}
-
-	ret = get_next_subbuffer_common(stream, subbuffer);
-	if (ret) {
-		goto end;
-	}
-end:
-	return ret;
-}
-
-static int get_next_subbuffer_metadata(struct lttng_consumer_stream *stream,
-		struct stream_subbuffer *subbuffer)
-{
-	int ret;
-	bool cache_empty;
-	bool got_subbuffer;
-	bool coherent;
-	bool buffer_empty;
-	unsigned long consumed_pos, produced_pos;
-
-	do {
-		ret = ustctl_get_next_subbuf(stream->ustream);
-		if (ret == 0) {
-			got_subbuffer = true;
-		} else {
-			got_subbuffer = false;
-			if (ret != -EAGAIN) {
-				/* Fatal error. */
-				goto end;
-			}
-		}
-
-		/*
-		 * Determine if the cache is empty and ensure that a sub-buffer
-		 * is made available if the cache is not empty.
-		 */
-		if (!got_subbuffer) {
-			ret = commit_one_metadata_packet(stream);
-			if (ret < 0 && ret != -ENOBUFS) {
-				goto end;
-			} else if (ret == 0) {
-				/* Not an error, the cache is empty. */
-				cache_empty = true;
-				ret = -ENODATA;
-				goto end;
-			} else {
-				cache_empty = false;
-			}
-		} else {
-			pthread_mutex_lock(&stream->chan->metadata_cache->lock);
-			cache_empty = stream->chan->metadata_cache->max_offset ==
-				      stream->ust_metadata_pushed;
-			pthread_mutex_unlock(&stream->chan->metadata_cache->lock);
-		}
-	} while (!got_subbuffer);
-
-	/* Populate sub-buffer infos and view. */
-	ret = get_next_subbuffer_common(stream, subbuffer);
-	if (ret) {
-		goto end;
-	}
-
-	ret = lttng_ustconsumer_sample_snapshot_positions(stream);
-	if (ret < 0) {
-		/*
-		 * -EAGAIN is not expected since we got a sub-buffer and haven't
-		 * pushed the consumption position yet (on put_next).
-		 */
-		PERROR("Failed to take a snapshot of metadata buffer positions");
-		goto end;
-	}
-
-	ret = lttng_ustconsumer_get_consumed_snapshot(stream, &consumed_pos);
-	if (ret) {
-		PERROR("Failed to get metadata consumed position");
-		goto end;
-	}
-
-	ret = lttng_ustconsumer_get_produced_snapshot(stream, &produced_pos);
-	if (ret) {
-		PERROR("Failed to get metadata produced position");
-		goto end;
-	}
-
-	/* Last sub-buffer of the ring buffer ? */
-	buffer_empty = (consumed_pos + stream->max_sb_size) == produced_pos;
-
 	/*
-	 * The sessiond registry lock ensures that coherent units of metadata
-	 * are pushed to the consumer daemon at once. Hence, if a sub-buffer is
-	 * acquired, the cache is empty, and it is the only available sub-buffer
-	 * available, it is safe to assume that it is "coherent".
+	 * If the stream was flagged to be ready for rotation before we extract the
+	 * next packet, rotate it now.
 	 */
-	coherent = got_subbuffer && cache_empty && buffer_empty;
-
-	LTTNG_OPTIONAL_SET(&subbuffer->info.metadata.coherent, coherent);
-end:
-	return ret;
-}
-
-static int put_next_subbuffer(struct lttng_consumer_stream *stream,
-		struct stream_subbuffer *subbuffer)
-{
-	const int ret = ustctl_put_next_subbuf(stream->ustream);
-
-	assert(ret == 0);
-	return ret;
-}
-
-static int signal_metadata(struct lttng_consumer_stream *stream,
-		struct lttng_consumer_local_data *ctx)
-{
-	return pthread_cond_broadcast(&stream->metadata_rdv) ? -errno : 0;
-}
-
-static int lttng_ustconsumer_set_stream_ops(
-		struct lttng_consumer_stream *stream)
-{
-	int ret = 0;
-
-	stream->read_subbuffer_ops.on_wake_up = consumer_stream_ust_on_wake_up;
-	if (stream->metadata_flag) {
-		stream->read_subbuffer_ops.get_next_subbuffer =
-				get_next_subbuffer_metadata;
-		stream->read_subbuffer_ops.extract_subbuffer_info =
-				extract_metadata_subbuffer_info;
-		stream->read_subbuffer_ops.reset_metadata =
-				metadata_stream_reset_cache_consumed_position;
-		if (stream->chan->is_live) {
-			stream->read_subbuffer_ops.on_sleep = signal_metadata;
-			ret = consumer_stream_enable_metadata_bucketization(
-					stream);
-			if (ret) {
-				goto end;
-			}
-		}
-	} else {
-		stream->read_subbuffer_ops.get_next_subbuffer =
-				get_next_subbuffer;
-		stream->read_subbuffer_ops.extract_subbuffer_info =
-				extract_data_subbuffer_info;
-		stream->read_subbuffer_ops.on_sleep = notify_if_more_data;
-		if (stream->chan->is_live) {
-			stream->read_subbuffer_ops.send_live_beacon =
-					consumer_flush_ust_index;
+	if (stream->rotate_ready) {
+		DBG("Rotate stream before extracting data");
+		rotation_ret = lttng_consumer_rotate_stream(ctx, stream);
+		if (rotation_ret < 0) {
+			ERR("Stream rotation error");
+			ret = -1;
+			goto error;
 		}
 	}
 
-	stream->read_subbuffer_ops.put_next_subbuffer = put_next_subbuffer;
-end:
+retry:
+	/* Get the next subbuffer */
+	err = ustctl_get_next_subbuf(ustream);
+	if (err != 0) {
+		/*
+		 * Populate metadata info if the existing info has
+		 * already been read.
+		 */
+		if (stream->metadata_flag) {
+			ret = commit_one_metadata_packet(stream);
+			if (ret <= 0) {
+				goto error;
+			}
+			goto retry;
+		}
+
+		ret = err;	/* ustctl_get_next_subbuf returns negative, caller expect positive. */
+		/*
+		 * This is a debug message even for single-threaded consumer,
+		 * because poll() have more relaxed criterions than get subbuf,
+		 * so get_subbuf may fail for short race windows where poll()
+		 * would issue wakeups.
+		 */
+		DBG("Reserving sub buffer failed (everything is normal, "
+				"it is due to concurrency) [ret: %d]", err);
+		goto error;
+	}
+	assert(stream->chan->output == CONSUMER_CHANNEL_MMAP);
+
+	if (!stream->metadata_flag) {
+		index.offset = htobe64(stream->out_fd_offset);
+		ret = get_index_values(&index, ustream);
+		if (ret < 0) {
+			err = ustctl_put_subbuf(ustream);
+			assert(err == 0);
+			goto error;
+		}
+
+		/* Update the stream's sequence and discarded events count. */
+		ret = update_stream_stats(stream);
+		if (ret < 0) {
+			PERROR("kernctl_get_events_discarded");
+			err = ustctl_put_subbuf(ustream);
+			assert(err == 0);
+			goto error;
+		}
+	} else {
+		write_index = 0;
+	}
+
+	/* Get the full padded subbuffer size */
+	err = ustctl_get_padded_subbuf_size(ustream, &len);
+	assert(err == 0);
+
+	/* Get subbuffer data size (without padding) */
+	err = ustctl_get_subbuf_size(ustream, &subbuf_size);
+	assert(err == 0);
+
+	/* Make sure we don't get a subbuffer size bigger than the padded */
+	assert(len >= subbuf_size);
+
+	padding = len - subbuf_size;
+
+	/* write the subbuffer to the tracefile */
+	ret = lttng_consumer_on_read_subbuffer_mmap(ctx, stream, subbuf_size, padding, &index);
+	/*
+	 * The mmap operation should write subbuf_size amount of data when network
+	 * streaming or the full padding (len) size when we are _not_ streaming.
+	 */
+	if ((ret != subbuf_size && stream->net_seq_idx != (uint64_t) -1ULL) ||
+			(ret != len && stream->net_seq_idx == (uint64_t) -1ULL)) {
+		/*
+		 * Display the error but continue processing to try to release the
+		 * subbuffer. This is a DBG statement since any unexpected kill or
+		 * signal, the application gets unregistered, relayd gets closed or
+		 * anything that affects the buffer lifetime will trigger this error.
+		 * So, for the sake of the user, don't print this error since it can
+		 * happen and it is OK with the code flow.
+		 */
+		DBG("Error writing to tracefile "
+				"(ret: %ld != len: %lu != subbuf_size: %lu)",
+				ret, len, subbuf_size);
+		write_index = 0;
+	}
+	err = ustctl_put_next_subbuf(ustream);
+	assert(err == 0);
+
+	/*
+	 * This will consumer the byte on the wait_fd if and only if there is not
+	 * next subbuffer to be acquired.
+	 */
+	if (!stream->metadata_flag) {
+		ret = notify_if_more_data(stream, ctx);
+		if (ret < 0) {
+			goto error;
+		}
+	}
+
+	/* Write index if needed. */
+	if (!write_index) {
+		goto rotate;
+	}
+
+	if (stream->chan->live_timer_interval && !stream->metadata_flag) {
+		/*
+		 * In live, block until all the metadata is sent.
+		 */
+		pthread_mutex_lock(&stream->metadata_timer_lock);
+		assert(!stream->missed_metadata_flush);
+		stream->waiting_on_metadata = true;
+		pthread_mutex_unlock(&stream->metadata_timer_lock);
+
+		err = consumer_stream_sync_metadata(ctx, stream->session_id);
+
+		pthread_mutex_lock(&stream->metadata_timer_lock);
+		stream->waiting_on_metadata = false;
+		if (stream->missed_metadata_flush) {
+			stream->missed_metadata_flush = false;
+			pthread_mutex_unlock(&stream->metadata_timer_lock);
+			(void) consumer_flush_ust_index(stream);
+		} else {
+			pthread_mutex_unlock(&stream->metadata_timer_lock);
+		}
+
+		if (err < 0) {
+			goto error;
+		}
+	}
+
+	assert(!stream->metadata_flag);
+	err = consumer_stream_write_index(stream, &index);
+	if (err < 0) {
+		goto error;
+	}
+
+rotate:
+	/*
+	 * After extracting the packet, we check if the stream is now ready to be
+	 * rotated and perform the action immediately.
+	 */
+	rotation_ret = lttng_consumer_stream_is_rotate_ready(stream);
+	if (rotation_ret == 1) {
+		rotation_ret = lttng_consumer_rotate_stream(ctx, stream);
+		if (rotation_ret < 0) {
+			ERR("Stream rotation error");
+			ret = -1;
+			goto error;
+		}
+	} else if (rotation_ret < 0) {
+		ERR("Checking if stream is ready to rotate");
+		ret = -1;
+		goto error;
+	}
+error:
 	return ret;
 }
 
@@ -2981,8 +3020,6 @@ int lttng_ustconsumer_on_recv_stream(struct lttng_consumer_stream *stream)
 			goto error;
 		}
 	}
-
-	lttng_ustconsumer_set_stream_ops(stream);
 	ret = 0;
 
 error:
