@@ -1,19 +1,8 @@
 /*
- * Copyright (c)  2011 David Goulet <david.goulet@polymtl.ca>
+ * Copyright (C) 2011 David Goulet <david.goulet@polymtl.ca>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * as published by the Free Software Foundation; only version 2
- * of the License.
+ * SPDX-License-Identifier: GPL-2.0-only
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <assert.h>
@@ -92,17 +81,17 @@ static void test_create_kernel_metadata(void)
 	ok(kern->metadata->fd == -1 &&
 	   kern->metadata->conf != NULL &&
 	   kern->metadata->conf->attr.overwrite
-			== DEFAULT_METADATA_OVERWRITE &&
+			== DEFAULT_CHANNEL_OVERWRITE &&
 	   kern->metadata->conf->attr.subbuf_size
 			== default_get_metadata_subbuf_size() &&
 	   kern->metadata->conf->attr.num_subbuf
 			== DEFAULT_METADATA_SUBBUF_NUM &&
 	   kern->metadata->conf->attr.switch_timer_interval
-			== DEFAULT_METADATA_SWITCH_TIMER &&
+			== DEFAULT_KERNEL_CHANNEL_SWITCH_TIMER &&
 	   kern->metadata->conf->attr.read_timer_interval
-			== DEFAULT_METADATA_READ_TIMER &&
+			== DEFAULT_KERNEL_CHANNEL_READ_TIMER &&
 	   kern->metadata->conf->attr.output
-			== LTTNG_EVENT_MMAP,
+	       == DEFAULT_KERNEL_CHANNEL_OUTPUT,
 	   "Validate kernel session metadata");
 
 	trace_kernel_destroy_metadata(kern->metadata);
@@ -202,5 +191,6 @@ int main(int argc, char **argv)
 	test_create_kernel_event();
 	test_create_kernel_stream();
 
-	return exit_status();
+	/* Success */
+	return 0;
 }
