@@ -1,28 +1,16 @@
-# Copyright (C) - 2012 David Goulet <dgoulet@efficios.com>
+# Copyright (C) 2012 David Goulet <dgoulet@efficios.com>
 #
-# This library is free software; you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the Free
-# Software Foundation; version 2.1 of the License.
+# SPDX-License-Identifier: LGPL-2.1-only
 #
-# This library is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
-# details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this library; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 
 SESSIOND_BIN="lttng-sessiond"
 SESSIOND_MATCH=".*lttng-sess.*"
-SESSIOND_PIDS=""
 RUNAS_BIN="lttng-runas"
 RUNAS_MATCH=".*lttng-runas.*"
 CONSUMERD_BIN="lttng-consumerd"
 CONSUMERD_MATCH=".*lttng-consumerd.*"
 RELAYD_BIN="lttng-relayd"
 RELAYD_MATCH=".*lttng-relayd.*"
-RELAYD_PIDS=""
 LTTNG_BIN="lttng"
 BABELTRACE_BIN="babeltrace"
 OUTPUT_DEST=/dev/null
@@ -48,22 +36,44 @@ export LTTNG_SESSIOND_PATH="/bin/true"
 
 source $TESTDIR/utils/tap/tap.sh
 
+if [ -z ${LTTNG_TEST_TEARDOWN_TIMEOUT+x} ]; then
+	LTTNG_TEST_TEARDOWN_TIMEOUT=60
+fi
+
 function full_cleanup ()
 {
-	if [ -n "${SESSIOND_PIDS}" ] || [ -n "${RELAYD_PIDS}" ]; then
-		kill -9 ${SESSIOND_PIDS} ${RELAYD_PIDS} > /dev/null 2>&1
-	fi
+	# Try to kill daemons gracefully
+	stop_lttng_relayd_notap SIGTERM $LTTNG_TEST_TEARDOWN_TIMEOUT
+	stop_lttng_sessiond_notap SIGTERM $LTTNG_TEST_TEARDOWN_TIMEOUT
+
+	# If daemons are still present, forcibly kill them
+	stop_lttng_relayd_notap SIGKILL $LTTNG_TEST_TEARDOWN_TIMEOUT
+	stop_lttng_sessiond_notap SIGKILL $LTTNG_TEST_TEARDOWN_TIMEOUT
+	stop_lttng_consumerd_notap SIGKILL $LTTNG_TEST_TEARDOWN_TIMEOUT
 
 	# Disable trap for SIGTERM since the following kill to the
 	# pidgroup will be SIGTERM. Otherwise it loops.
 	# The '-' before the pid number ($$) indicates 'kill' to signal the
 	# whole process group.
 	trap - SIGTERM && kill -- -$$
+	exit 1
 }
 
+function null_pipes ()
+{
+	exec 0>/dev/null
+	exec 1>/dev/null
+	exec 2>/dev/null
+}
 
 trap full_cleanup SIGINT SIGTERM
 
+# perl prove closes its child pipes before giving it a chance to run its
+# signal trap handlers. Redirect pipes to /dev/null if SIGPIPE is caught
+# to allow those trap handlers to proceed.
+
+trap null_pipes SIGPIPE
+
 function print_ok ()
 {
 	# Check if we are a terminal
@@ -125,12 +135,23 @@ function conf_proc_count()
 	echo
 }
 
+# Check if base lttng-modules are present.
+# Bail out on failure
+function validate_lttng_modules_present ()
+{
+	modprobe -n lttng-tracer 2>/dev/null
+	if [ $? -ne 0  ]; then
+		BAIL_OUT "LTTng modules not detected."
+	fi
+}
+
 function enable_kernel_lttng_event
 {
-	local expected_to_fail="$1"
-	local sess_name="$2"
-	local event_name="$3"
-	local channel_name="$4"
+	local withtap="$1"
+	local expected_to_fail="$2"
+	local sess_name="$3"
+	local event_name="$4"
+	local channel_name="$5"
 
 	if [ -z "$event_name" ]; then
 		# Enable all event if no event name specified
@@ -148,20 +169,30 @@ function enable_kernel_lttng_event
 	ret=$?
 	if [[ $expected_to_fail -eq "1" ]]; then
 		test $ret -ne "0"
-		ok $? "Enable kernel event $event_name for session $session_name on channel $channel_name failed as expected"
+		ret=$?
+		if [ $withtap -eq "1" ]; then
+			ok $ret "Enable kernel event $event_name for session $session_name on channel $channel_name failed as expected"
+		fi
 	else
-		ok $ret "Enable kernel event $event_name for session $sess_name"
+		if [ $withtap -eq "1" ]; then
+			ok $ret "Enable kernel event $event_name for session $sess_name"
+		fi
 	fi
 }
 
 function enable_kernel_lttng_event_ok ()
 {
-	enable_kernel_lttng_event 0 "$@"
+	enable_kernel_lttng_event 1 0 "$@"
 }
 
 function enable_kernel_lttng_event_fail ()
 {
-	enable_kernel_lttng_event 1 "$@"
+	enable_kernel_lttng_event 1 1 "$@"
+}
+
+function enable_kernel_lttng_event_notap ()
+{
+	enable_kernel_lttng_event 0 0 "$@"
 }
 
 # Old interface
@@ -249,30 +280,82 @@ function lttng_disable_kernel_syscall_fail()
 	lttng_disable_kernel_syscall 1 "$@"
 }
 
+function lttng_enable_kernel_userspace_probe_event ()
+{
+	local expected_to_fail="$1"
+	local sess_name="$2"
+	local target="$3"
+	local event_name="$4"
+
+	"$TESTDIR/../src/bin/lttng/$LTTNG_BIN" enable-event --kernel --userspace-probe="$target" "$event_name" -s "$sess_name" > "$OUTPUT_DEST" 2> "$ERROR_OUTPUT_DEST"
+	ret=$?
+	if [[ $expected_to_fail -eq "1" ]]; then
+		test $ret -ne "0"
+		ok $? "Enable kernel userspace probe event for session $sess_name failed as expected"
+	else
+		ok $ret "Enable kernel userspace probe event for session $sess_name"
+	fi
+}
+
+function lttng_enable_kernel_userspace_probe_event_fail ()
+{
+	lttng_enable_kernel_userspace_probe_event 1 "$@"
+}
+
+function lttng_enable_kernel_userspace_probe_event_ok ()
+{
+	lttng_enable_kernel_userspace_probe_event 0 "$@"
+}
+
+function disable_kernel_lttng_userspace_probe_event_ok ()
+{
+	local sess_name="$1"
+	local event_name="$2"
+
+	"$TESTDIR/../src/bin/lttng/$LTTNG_BIN" disable-event --kernel "$event_name" -s "$sess_name" > "$OUTPUT_DEST" 2> "$ERROR_OUTPUT_DEST"
+	ok $? "Disable kernel event $target for session $sess_name"
+}
 function lttng_enable_kernel_channel()
 {
-	local expected_to_fail=$1
-	local sess_name=$2
-	local channel_name=$3
+	local withtap=$1
+	local expected_to_fail=$2
+	local sess_name=$3
+	local channel_name=$4
+	local opts="${@:5}"
 
-	$TESTDIR/../src/bin/lttng/$LTTNG_BIN enable-channel -k $channel_name -s $sess_name 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN enable-channel -k $channel_name -s $sess_name $opts 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
 	ret=$?
 	if [[ $expected_to_fail -eq "1" ]]; then
 		test "$ret" -ne "0"
-		ok $? "Enable channel $channel_name for session $sess_name failed as expected"
+		ret=$?
+		if [ $withtap -eq "1" ]; then
+			ok $ret "Enable channel $channel_name for session $sess_name failed as expected"
+		fi
 	else
-		ok $ret "Enable channel $channel_name for session $sess_name"
+		if [ $withtap -eq "1" ]; then
+			ok $ret "Enable channel $channel_name for session $sess_name"
+		fi
 	fi
 }
 
 function lttng_enable_kernel_channel_ok()
 {
-	lttng_enable_kernel_channel 0 "$@"
+	lttng_enable_kernel_channel 1 0 "$@"
 }
 
 function lttng_enable_kernel_channel_fail()
 {
-	lttng_enable_kernel_channel 1 "$@"
+	lttng_enable_kernel_channel 1 1 "$@"
+}
+
+function lttng_enable_kernel_channel_notap()
+{
+	lttng_enable_kernel_channel 0 0 "$@"
+}
+
+function enable_kernel_lttng_channel_ok()
+{
+	lttng_enable_kernel_channel 1 0 "$@"
 }
 
 function lttng_disable_kernel_channel()
@@ -304,66 +387,97 @@ function lttng_disable_kernel_channel_fail()
 function start_lttng_relayd_opt()
 {
 	local withtap=$1
-	local opt=$2
+	local process_mode=$2
+	local opt=$3
 
-	DIR=$(readlink -f $TESTDIR)
+	DIR=$(readlink -f "$TESTDIR")
 
 	if [ -z $(pgrep $RELAYD_MATCH) ]; then
-		$DIR/../src/bin/lttng-relayd/$RELAYD_BIN -b $opt 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+		# shellcheck disable=SC2086
+		$DIR/../src/bin/lttng-relayd/$RELAYD_BIN $process_mode $opt 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
 		#$DIR/../src/bin/lttng-relayd/$RELAYD_BIN $opt -vvv >>/tmp/relayd.log 2>&1 &
 		if [ $? -eq 1 ]; then
 			if [ $withtap -eq "1" ]; then
-				fail "Start lttng-relayd (opt: $opt)"
+				fail "Start lttng-relayd (process mode: $process_mode opt: $opt)"
 			fi
 			return 1
 		else
 			if [ $withtap -eq "1" ]; then
-				pass "Start lttng-relayd (opt: $opt)"
+				pass "Start lttng-relayd (process mode: $process_mode opt: $opt)"
 			fi
 		fi
 	else
 		pass "Start lttng-relayd (opt: $opt)"
 	fi
-
-	RELAYD_PIDS=$(pgrep $RELAYD_MATCH)
 }
 
 function start_lttng_relayd()
 {
-	start_lttng_relayd_opt 1 "$@"
+	start_lttng_relayd_opt 1 "-b" "$@"
 }
 
 function start_lttng_relayd_notap()
 {
-	start_lttng_relayd_opt 0 "$@"
+	start_lttng_relayd_opt 0 "-b" "$@"
 }
 
 function stop_lttng_relayd_opt()
 {
 	local withtap=$1
+	local signal=$2
 
-	if [ $withtap -eq "1" ]; then
-		diag "Killing lttng-relayd (pid: $RELAYD_PIDS)"
+	if [ -z "$signal" ]; then
+		signal="SIGTERM"
 	fi
-	kill $RELAYD_PIDS 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
-	retval=$?
 
-	if [ $? -eq 1 ]; then
-		if [ $withtap -eq "1" ]; then
+	local timeout_s=$3
+	local dtimeleft_s=
+
+	# Multiply time by 2 to simplify integer arithmetic
+	if [ -n "$timeout_s" ]; then
+		dtimeleft_s=$((timeout_s * 2))
+	fi
+
+	local retval=0
+	local pids=
+
+	pids=$(pgrep "$RELAYD_MATCH")
+	if [ -z "$pids" ]; then
+		if [ "$withtap" -eq "1" ]; then
+			pass "No relay daemon to kill"
+		fi
+		return 0
+	fi
+
+	diag "Killing (signal $signal) lttng-relayd (pid: $pids)"
+
+	# shellcheck disable=SC2086
+	if ! kill -s $signal $pids 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST; then
+		retval=1
+		if [ "$withtap" -eq "1" ]; then
 			fail "Kill relay daemon"
 		fi
-		return 1
 	else
 		out=1
 		while [ -n "$out" ]; do
-			out=$(pgrep $RELAYD_MATCH)
+			out=$(pgrep "$RELAYD_MATCH")
+			if [ -n "$dtimeleft_s" ]; then
+				if [ $dtimeleft_s -lt 0 ]; then
+					out=
+					retval=1
+				fi
+				dtimeleft_s=$((dtimeleft_s - 1))
+			fi
 			sleep 0.5
 		done
-		if [ $withtap -eq "1" ]; then
-			pass "Kill relay daemon"
+		if [ "$withtap" -eq "1" ]; then
+			if [ "$retval" -eq "0" ]; then
+				pass "Wait after kill relay daemon"
+			else
+				fail "Wait after kill relay daemon"
+			fi
 		fi
 	fi
-	RELAYD_PIDS=""
 	return $retval
 }
 
@@ -384,35 +498,61 @@ function start_lttng_sessiond_opt()
 	local withtap=$1
 	local load_path=$2
 
-	if [ -n $TEST_NO_SESSIOND ] && [ "$TEST_NO_SESSIOND" == "1" ]; then
+	local env_vars=""
+	local consumerd=""
+
+	local long_bit_value=
+	long_bit_value=$(getconf LONG_BIT)
+
+	if [ -n "$TEST_NO_SESSIOND" ] && [ "$TEST_NO_SESSIOND" == "1" ]; then
 		# Env variable requested no session daemon
 		return
 	fi
 
-	validate_kernel_version
-	if [ $? -ne 0 ]; then
+	DIR=$(readlink -f "$TESTDIR")
+
+	# Get long_bit value for 32/64 consumerd
+	case "$long_bit_value" in
+		32)
+			consumerd="--consumerd32-path=$DIR/../src/bin/lttng-consumerd/lttng-consumerd"
+			;;
+		64)
+			consumerd="--consumerd64-path=$DIR/../src/bin/lttng-consumerd/lttng-consumerd"
+			;;
+		*)
+			return
+			;;
+	esac
+
+	# Check for env. variable. Allow the use of LD_PRELOAD etc.
+	if [[ "x${LTTNG_SESSIOND_ENV_VARS}" != "x" ]]; then
+		env_vars="${LTTNG_SESSIOND_ENV_VARS} "
+	fi
+	env_vars="${env_vars}$DIR/../src/bin/lttng-sessiond/$SESSIOND_BIN"
+
+	if ! validate_kernel_version; then
 	    fail "Start session daemon"
 	    BAIL_OUT "*** Kernel too old for session daemon tests ***"
 	fi
 
-	DIR=$(readlink -f $TESTDIR)
-	: ${LTTNG_SESSION_CONFIG_XSD_PATH=${DIR}/../src/common/config/}
+	: "${LTTNG_SESSION_CONFIG_XSD_PATH="${DIR}/../src/common/config/"}"
 	export LTTNG_SESSION_CONFIG_XSD_PATH
 
-	if [ -z $(pgrep ${SESSIOND_MATCH}) ]; then
+	if [ -z "$(pgrep "${SESSIOND_MATCH}")" ]; then
 		# Have a load path ?
 		if [ -n "$load_path" ]; then
-			$DIR/../src/bin/lttng-sessiond/$SESSIOND_BIN --load "$load_path" --background --consumerd32-path="$DIR/../src/bin/lttng-consumerd/lttng-consumerd" --consumerd64-path="$DIR/../src/bin/lttng-consumerd/lttng-consumerd"
+			# shellcheck disable=SC2086
+			env $env_vars --load "$load_path" --background "$consumerd"
 		else
-			$DIR/../src/bin/lttng-sessiond/$SESSIOND_BIN --background --consumerd32-path="$DIR/../src/bin/lttng-consumerd/lttng-consumerd" --consumerd64-path="$DIR/../src/bin/lttng-consumerd/lttng-consumerd"
+			# shellcheck disable=SC2086
+			env $env_vars --background "$consumerd"
 		fi
 		#$DIR/../src/bin/lttng-sessiond/$SESSIOND_BIN --background --consumerd32-path="$DIR/../src/bin/lttng-consumerd/lttng-consumerd" --consumerd64-path="$DIR/../src/bin/lttng-consumerd/lttng-consumerd" --verbose-consumer >>/tmp/sessiond.log 2>&1
 		status=$?
-		if [ $withtap -eq "1" ]; then
+		if [ "$withtap" -eq "1" ]; then
 			ok $status "Start session daemon"
 		fi
 	fi
-	SESSIOND_PIDS=$(pgrep $SESSIOND_MATCH)
 }
 
 function start_lttng_sessiond()
@@ -429,44 +569,98 @@ function stop_lttng_sessiond_opt()
 {
 	local withtap=$1
 	local signal=$2
-	local kill_opt=""
 
-	if [ -n $TEST_NO_SESSIOND ] && [ "$TEST_NO_SESSIOND" == "1" ]; then
+	if [ -z "$signal" ]; then
+		signal=SIGTERM
+	fi
+
+	local timeout_s=$3
+	local dtimeleft_s=
+
+	# Multiply time by 2 to simplify integer arithmetic
+	if [ -n "$timeout_s" ]; then
+		dtimeleft_s=$((timeout_s * 2))
+	fi
+
+	if [ -n "$TEST_NO_SESSIOND" ] && [ "$TEST_NO_SESSIOND" == "1" ]; then
 		# Env variable requested no session daemon
-		return
+		return 0
 	fi
 
-	local pids="${SESSIOND_PIDS} $(pgrep $RUNAS_MATCH)"
+	local retval=0
 
-	if [ -n "$2" ]; then
-		kill_opt="$kill_opt -s $signal"
-	fi
-	if [ $withtap -eq "1" ]; then
-		diag "Killing $SESSIOND_BIN and lt-$SESSIOND_BIN pids: $(echo $pids | tr '\n' ' ')"
-	fi
-	kill $kill_opt $pids 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	local runas_pids=
+	runas_pids=$(pgrep "$RUNAS_MATCH")
 
-	if [ $? -eq 1 ]; then
-		if [ $withtap -eq "1" ]; then
+	local pids=
+	pids=$(pgrep "$SESSIOND_MATCH")
+
+	if [ -n "$runas_pids" ]; then
+		pids="$pids $runas_pids"
+	fi
+
+	if [ -z "$pids" ]; then
+		if [ "$withtap" -eq "1" ]; then
+			pass "No session daemon to kill"
+		fi
+		return 0
+	fi
+
+	diag "Killing (signal $signal) $SESSIOND_BIN and lt-$SESSIOND_BIN pids: $(echo "$pids" | tr '\n' ' ')"
+
+	# shellcheck disable=SC2086
+	if ! kill -s $signal $pids 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST; then
+		retval=1
+		if [ "$withtap" -eq "1" ]; then
 			fail "Kill sessions daemon"
 		fi
 	else
 		out=1
 		while [ -n "$out" ]; do
-			out=$(pgrep ${SESSIOND_MATCH})
+			out=$(pgrep "${SESSIOND_MATCH}")
+			if [ -n "$dtimeleft_s" ]; then
+				if [ $dtimeleft_s -lt 0 ]; then
+					out=
+					retval=1
+				fi
+				dtimeleft_s=$((dtimeleft_s - 1))
+			fi
 			sleep 0.5
 		done
 		out=1
 		while [ -n "$out" ]; do
-			out=$(pgrep $CONSUMERD_MATCH)
+			out=$(pgrep "$CONSUMERD_MATCH")
+			if [ -n "$dtimeleft_s" ]; then
+				if [ $dtimeleft_s -lt 0 ]; then
+					out=
+					retval=1
+				fi
+				dtimeleft_s=$((dtimeleft_s - 1))
+			fi
 			sleep 0.5
 		done
 
-		SESSIOND_PIDS=""
-		if [ $withtap -eq "1" ]; then
-			pass "Kill session daemon"
+		if [ "$withtap" -eq "1" ]; then
+			if [ "$retval" -eq "0" ]; then
+				pass "Wait after kill session daemon"
+			else
+				fail "Wait after kill session daemon"
+			fi
 		fi
 	fi
+	if [ "$signal" = "SIGKILL" ]; then
+		if [ "$(id -u)" -eq "0" ]; then
+			local modules=
+			modules="$(lsmod | grep ^lttng | awk '{print $1}')"
+
+			if [ -n "$modules" ]; then
+				diag "Unloading all LTTng modules"
+				modprobe -r "$modules"
+			fi
+		fi
+	fi
+
+	return $retval
 }
 
 function stop_lttng_sessiond()
@@ -483,43 +677,40 @@ function sigstop_lttng_sessiond_opt()
 {
 	local withtap=$1
 	local signal=SIGSTOP
-	local kill_opt=""
 
-	if [ -n $TEST_NO_SESSIOND ] && [ "$TEST_NO_SESSIOND" == "1" ]; then
+	if [ -n "$TEST_NO_SESSIOND" ] && [ "$TEST_NO_SESSIOND" == "1" ]; then
 		# Env variable requested no session daemon
 		return
 	fi
 
-	PID_SESSIOND="$(pgrep ${SESSIOND_MATCH}) $(pgrep $RUNAS_MATCH)"
+	PID_SESSIOND="$(pgrep "${SESSIOND_MATCH}") $(pgrep "$RUNAS_MATCH")"
 
-	kill_opt="$kill_opt -s $signal"
-
-	if [ $withtap -eq "1" ]; then
-		diag "Sending SIGSTOP to lt-$SESSIOND_BIN and $SESSIOND_BIN pids: $(echo $PID_SESSIOND | tr '\n' ' ')"
+	if [ "$withtap" -eq "1" ]; then
+		diag "Sending SIGSTOP to lt-$SESSIOND_BIN and $SESSIOND_BIN pids: $(echo "$PID_SESSIOND" | tr '\n' ' ')"
 	fi
-	kill $kill_opt $PID_SESSIOND 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
 
-	if [ $? -eq 1 ]; then
-		if [ $withtap -eq "1" ]; then
+	# shellcheck disable=SC2086
+	if ! kill -s $signal $PID_SESSIOND 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST; then
+		if [ "$withtap" -eq "1" ]; then
 			fail "Sending SIGSTOP to session daemon"
 		fi
 	else
 		out=1
 		while [ $out -ne 0 ]; do
-			pid=$(pgrep $SESSIOND_MATCH)
+			pid="$(pgrep "$SESSIOND_MATCH")"
 
 			# Wait until state becomes stopped for session
 			# daemon(s).
 			out=0
 			for sessiond_pid in $pid; do
-				state=$(ps -p $sessiond_pid -o state= )
+				state="$(ps -p "$sessiond_pid" -o state= )"
 				if [[ -n "$state" && "$state" != "T" ]]; then
 					out=1
 				fi
 			done
 			sleep 0.5
 		done
-		if [ $withtap -eq "1" ]; then
+		if [ "$withtap" -eq "1" ]; then
 			pass "Sending SIGSTOP to session daemon"
 		fi
 	fi
@@ -539,46 +730,70 @@ function stop_lttng_consumerd_opt()
 {
 	local withtap=$1
 	local signal=$2
-	local kill_opt=""
 
-	PID_CONSUMERD=$(pgrep $CONSUMERD_MATCH)
-
-	if [ -n "$2" ]; then
-		kill_opt="$kill_opt -s $signal"
+	if [ -z "$signal" ]; then
+		signal=SIGTERM
 	fi
 
-	if [ $withtap -eq "1" ]; then
-		diag "Killing $CONSUMERD_BIN pids: $(echo $PID_CONSUMERD | tr '\n' ' ')"
+	local timeout_s=$3
+	local dtimeleft_s=
+
+	# Multiply time by 2 to simplify integer arithmetic
+	if [ -n "$timeout_s" ]; then
+		dtimeleft_s=$((timeout_s * 2))
 	fi
 
-	kill $kill_opt $PID_CONSUMERD 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
-	retval=$?
+	local retval=0
 
-	if [ $? -eq 1 ]; then
-		if [ $withtap -eq "1" ]; then
+	PID_CONSUMERD="$(pgrep "$CONSUMERD_MATCH")"
+
+	if [ -z "$PID_CONSUMERD" ]; then
+		if [ "$withtap" -eq "1" ]; then
+			pass "No consumer daemon to kill"
+		fi
+		return 0
+	fi
+
+	diag "Killing (signal $signal) $CONSUMERD_BIN pids: $(echo "$PID_CONSUMERD" | tr '\n' ' ')"
+
+	# shellcheck disable=SC2086
+	if ! kill -s $signal $PID_CONSUMERD 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST; then
+		retval=1
+		if [ "$withtap" -eq "1" ]; then
 			fail "Kill consumer daemon"
 		fi
-		return 1
 	else
 		out=1
 		while [ $out -ne 0 ]; do
-			pid=$(pgrep $CONSUMERD_MATCH)
+			pid="$(pgrep "$CONSUMERD_MATCH")"
 
 			# If consumerds are still present check their status.
 			# A zombie status qualifies the consumerd as *killed*
 			out=0
 			for consumer_pid in $pid; do
-				state=$(ps -p $consumer_pid -o state= )
+				state="$(ps -p "$consumer_pid" -o state= )"
 				if [[ -n "$state" && "$state" != "Z" ]]; then
 					out=1
 				fi
 			done
+			if [ -n "$dtimeleft_s" ]; then
+				if [ $dtimeleft_s -lt 0 ]; then
+					out=0
+					retval=1
+				fi
+				dtimeleft_s=$((dtimeleft_s - 1))
+			fi
 			sleep 0.5
 		done
-		if [ $withtap -eq "1" ]; then
-			pass "Kill consumer daemon"
+		if [ "$withtap" -eq "1" ]; then
+			if [ "$retval" -eq "0" ]; then
+				pass "Wait after kill consumer daemon"
+			else
+				fail "Wait after kill consumer daemon"
+			fi
 		fi
 	fi
+
 	return $retval
 }
 
@@ -596,40 +811,37 @@ function sigstop_lttng_consumerd_opt()
 {
 	local withtap=$1
 	local signal=SIGSTOP
-	local kill_opt=""
 
-	PID_CONSUMERD=$(pgrep $CONSUMERD_MATCH)
+	PID_CONSUMERD="$(pgrep "$CONSUMERD_MATCH")"
 
-	kill_opt="$kill_opt -s $signal"
+	diag "Sending SIGSTOP to $CONSUMERD_BIN pids: $(echo "$PID_CONSUMERD" | tr '\n' ' ')"
 
-	if [ $withtap -eq "1" ]; then
-		diag "Sending SIGSTOP to $CONSUMERD_BIN pids: $(echo $PID_CONSUMERD | tr '\n' ' ')"
-	fi
-	kill $kill_opt $PID_CONSUMERD 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	# shellcheck disable=SC2086
+	kill -s $signal $PID_CONSUMERD 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
 	retval=$?
 
-	if [ $? -eq 1 ]; then
-		if [ $withtap -eq "1" ]; then
+	if [ $retval -eq 1 ]; then
+		if [ "$withtap" -eq "1" ]; then
 			fail "Sending SIGSTOP to consumer daemon"
 		fi
 		return 1
 	else
 		out=1
 		while [ $out -ne 0 ]; do
-			pid=$(pgrep $CONSUMERD_MATCH)
+			pid="$(pgrep "$CONSUMERD_MATCH")"
 
 			# Wait until state becomes stopped for all
 			# consumers.
 			out=0
 			for consumer_pid in $pid; do
-				state=$(ps -p $consumer_pid -o state= )
+				state="$(ps -p "$consumer_pid" -o state= )"
 				if [[ -n "$state" && "$state" != "T" ]]; then
 					out=1
 				fi
 			done
 			sleep 0.5
 		done
-		if [ $withtap -eq "1" ]; then
+		if [ "$withtap" -eq "1" ]; then
 			pass "Sending SIGSTOP to consumer daemon"
 		fi
 	fi
@@ -656,64 +868,105 @@ function list_lttng_with_opts ()
 function create_lttng_session_no_output ()
 {
 	local sess_name=$1
+	local opts="${@:2}"
 
-	$TESTDIR/../src/bin/lttng/$LTTNG_BIN create $sess_name --no-output 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN create $sess_name --no-output $opts 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
 	ok $? "Create session $sess_name in no-output mode"
 }
 
+function create_lttng_session_uri () {
+	local sess_name=$1
+	local uri=$2
+	local opts="${@:3}"
+
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN create $sess_name -U $uri $opts 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	ok $? "Create session $sess_name with uri:$uri and opts: $opts"
+}
+
 function create_lttng_session ()
 {
-	local expected_to_fail=$1
-	local sess_name=$2
-	local trace_path=$3
-	local opt=$4
+	local withtap=$1
+	local expected_to_fail=$2
+	local sess_name=$3
+	local trace_path=$4
+	local opt=$5
 
-	$TESTDIR/../src/bin/lttng/$LTTNG_BIN create $sess_name -o $trace_path $opt > $OUTPUT_DEST
-	ret=$?
-	if [[ $expected_to_fail -eq "1" ]]; then
-		test "$ret" -ne "0"
-		ok $? "Create session $sess_name in $trace_path failed as expected"
+	if [ -z "$trace_path" ]; then
+		# Use lttng-sessiond default output.
+		trace_path=""
 	else
-		ok $ret "Create session $sess_name in $trace_path"
+		trace_path="-o $trace_path"
 	fi
+
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN create "$sess_name" $trace_path $opt 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	ret=$?
+	if [ $expected_to_fail -eq "1" ]; then
+		test "$ret" -ne "0"
+		ret=$?
+		if [ $withtap -eq "1" ]; then
+			ok $ret "Create session $sess_name in $trace_path failed as expected"
+		fi
+	else
+		if [ $withtap -eq "1" ]; then
+			ok $ret "Create session $sess_name in $trace_path"
+		fi
+	fi
+	return $ret
 }
 
 function create_lttng_session_ok ()
 {
-	create_lttng_session 0 "$@"
+	create_lttng_session 1 0 "$@"
 }
 
 function create_lttng_session_fail ()
 {
-	create_lttng_session 1 "$@"
+	create_lttng_session 1 1 "$@"
+}
+
+function create_lttng_session_notap ()
+{
+	create_lttng_session 0 0 "$@"
 }
 
 
 function enable_ust_lttng_channel ()
 {
-	local expected_to_fail=$1
-	local sess_name=$2
-	local channel_name=$3
-	local opt=$4
+	local withtap=$1
+	local expected_to_fail=$2
+	local sess_name=$3
+	local channel_name=$4
+	local opts="${@:5}"
 
-	$TESTDIR/../src/bin/lttng/$LTTNG_BIN enable-channel -u $channel_name -s $sess_name $opt 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN enable-channel -u $channel_name -s $sess_name $opts 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
 	ret=$?
 	if [[ $expected_to_fail -eq "1" ]]; then
 		test "$ret" -ne "0"
-		ok $? "Enable channel $channel_name for session $sess_name failed as expected"
+		ret=$?
+		if [ $withtap -eq "1" ]; then
+			ok $ret "Enable channel $channel_name for session $sess_name failed as expected"
+		fi
 	else
-		ok $ret "Enable channel $channel_name for session $sess_name"
+		if [ $withtap -eq "1" ]; then
+			ok $ret "Enable channel $channel_name for session $sess_name"
+		fi
 	fi
+	return $ret
 }
 
 function enable_ust_lttng_channel_ok ()
 {
-	enable_ust_lttng_channel 0 "$@"
+	enable_ust_lttng_channel 1 0 "$@"
 }
 
 function enable_ust_lttng_channel_fail ()
 {
-	enable_ust_lttng_channel 1 "$@"
+	enable_ust_lttng_channel 1 1 "$@"
+}
+
+function enable_ust_lttng_channel_notap ()
+{
+	enable_ust_lttng_channel 0 0 "$@"
 }
 
 function disable_ust_lttng_channel()
@@ -763,10 +1016,11 @@ function enable_lttng_mmap_overwrite_ust_channel()
 
 function enable_ust_lttng_event ()
 {
-	local expected_to_fail=$1
-	local sess_name=$2
-	local event_name="$3"
-	local channel_name=$4
+	local withtap=$1
+	local expected_to_fail=$2
+	local sess_name=$3
+	local event_name="$4"
+	local channel_name=$5
 
 	if [ -z $channel_name ]; then
 		# default channel if none specified
@@ -779,20 +1033,31 @@ function enable_ust_lttng_event ()
 	ret=$?
 	if [[ $expected_to_fail -eq "1" ]]; then
 		test $ret -ne "0"
-		ok $? "Enable ust event $event_name for session $session_name failed as expected"
+		ret=$?
+		if [[ $withtap -eq "1" ]]; then
+			ok $ret "Enable ust event $event_name for session $session_name failed as expected"
+		fi
 	else
-		ok $ret "Enable ust event $event_name for session $sess_name"
+		if [[ $withtap -eq "1" ]]; then
+			ok $ret "Enable ust event $event_name for session $sess_name"
+		fi
 	fi
+	return $ret
 }
 
 function enable_ust_lttng_event_ok ()
 {
-	enable_ust_lttng_event 0 "$@"
+	enable_ust_lttng_event 1 0 "$@"
 }
 
 function enable_ust_lttng_event_fail ()
 {
-	enable_ust_lttng_event 1 "$@"
+	enable_ust_lttng_event 1 1 "$@"
+}
+
+function enable_ust_lttng_event_notap ()
+{
+	enable_ust_lttng_event 0 0 "$@"
 }
 
 function enable_jul_lttng_event()
@@ -974,82 +1239,114 @@ function disable_python_lttng_event ()
 	ok $? "Disable Python event $event_name for session $sess_name"
 }
 
-function start_lttng_tracing ()
+function start_lttng_tracing_opt ()
 {
-	local expected_to_fail=$1
-	local sess_name=$2
+	local withtap=$1
+	local expected_to_fail=$2
+	local sess_name=$3
 
 	$TESTDIR/../src/bin/lttng/$LTTNG_BIN start $sess_name 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
 	ret=$?
 	if [[ $expected_to_fail -eq "1" ]]; then
 		test "$ret" -ne "0"
-		ok $? "Start tracing for session $sess_name failed as expected"
+		ret=$?
+		if [ $withtap -eq "1" ]; then
+			ok $? "Start tracing for session $sess_name failed as expected"
+		fi
 	else
-		ok $ret "Start tracing for session $sess_name"
+		if [ $withtap -eq "1" ]; then
+			ok $ret "Start tracing for session $sess_name"
+		fi
 	fi
 }
 
 function start_lttng_tracing_ok ()
 {
-	start_lttng_tracing 0 "$@"
+	start_lttng_tracing_opt 1 0 "$@"
 }
 
 function start_lttng_tracing_fail ()
 {
-	start_lttng_tracing 1 "$@"
+	start_lttng_tracing_opt 1 1 "$@"
 }
 
-function stop_lttng_tracing ()
+function start_lttng_tracing_notap ()
 {
-	local expected_to_fail=$1
-	local sess_name=$2
+	start_lttng_tracing_opt 0 1 "$@"
+}
+
+function stop_lttng_tracing_opt ()
+{
+	local withtap=$1
+	local expected_to_fail=$2
+	local sess_name=$3
 
 	$TESTDIR/../src/bin/lttng/$LTTNG_BIN stop $sess_name 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
 	ret=$?
 	if [[ $expected_to_fail -eq "1" ]]; then
 		test "$ret" -ne "0"
-		ok $? "Stop lttng tracing for session $sess_name failed as expected"
+		ret=$?
+		if [ $withtap -eq "1" ]; then
+			ok $? "Stop lttng tracing for session $sess_name failed as expected"
+		fi
 	else
-		ok $ret "Stop lttng tracing for session $sess_name"
+		if [ $withtap -eq "1" ]; then
+			ok $ret "Stop lttng tracing for session $sess_name"
+		fi
 	fi
 }
 
 function stop_lttng_tracing_ok ()
 {
-	stop_lttng_tracing 0 "$@"
+	stop_lttng_tracing_opt 1 0 "$@"
 }
 
 function stop_lttng_tracing_fail ()
 {
-	stop_lttng_tracing 1 "$@"
+	stop_lttng_tracing_opt 1 1 "$@"
+}
+
+function stop_lttng_tracing_notap ()
+{
+	stop_lttng_tracing_opt 0 0 "$@"
 }
 
 function destroy_lttng_session ()
 {
-	local expected_to_fail=$1
-	local sess_name=$2
+	local withtap=$1
+	local expected_to_fail=$2
+	local sess_name=$3
 
 	$TESTDIR/../src/bin/lttng/$LTTNG_BIN destroy $sess_name 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
 	ret=$?
 	if [[ $expected_to_fail -eq "1" ]]; then
 		test "$ret" -ne "0"
-		ok $? "Destroy session $sess_name failed as expected"
+		ret=$?
+		if [ $withtap -eq "1" ]; then
+			ok $ret "Destroy session $sess_name failed as expected"
+		fi
 	else
-		ok $ret "Destroy session $sess_name"
+		if [ $withtap -eq "1" ]; then
+			ok $ret "Destroy session $sess_name"
+		fi
 	fi
 }
 
 function destroy_lttng_session_ok ()
 {
-	destroy_lttng_session 0 "$@"
+	destroy_lttng_session 1 0 "$@"
 
 }
 
 function destroy_lttng_session_fail ()
 {
-	destroy_lttng_session 1 "$@"
+	destroy_lttng_session 1 1 "$@"
 }
 
+function destroy_lttng_session_notap ()
+{
+	destroy_lttng_session 0 0 "$@"
+}
 
 function destroy_lttng_sessions ()
 {
@@ -1062,14 +1359,15 @@ function lttng_snapshot_add_output ()
 	local expected_to_fail=$1
 	local sess_name=$2
 	local trace_path=$3
+	local opts=$4
 
-	$TESTDIR/../src/bin/lttng/$LTTNG_BIN snapshot add-output -s $sess_name file://$trace_path 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN snapshot add-output -s $sess_name $trace_path $opts 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
 	ret=$?
 	if [[ $expected_to_fail -eq 1 ]]; then
 		test "$ret" -ne "0"
-		ok $? "Added snapshot output file://$trace_path failed as expected"
+		ok $? "Added snapshot output $trace_path failed as expected"
 	else
-		ok $ret "Added snapshot output file://$trace_path"
+		ok $ret "Added snapshot output $trace_path"
 	fi
 }
 
@@ -1112,9 +1410,8 @@ function lttng_snapshot_del_output_fail ()
 function lttng_snapshot_record ()
 {
 	local sess_name=$1
-	local trace_path=$2
 
-	$TESTDIR/../src/bin/lttng/$LTTNG_BIN snapshot record -s $sess_name 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN snapshot record -s $sess_name $trace_path 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
 	ok $? "Snapshot recorded"
 }
 
@@ -1161,8 +1458,9 @@ function lttng_load_fail()
 
 function lttng_track()
 {
-	local expected_to_fail=$1
-	local opts=$2
+	local expected_to_fail="$1"
+	shift 1
+	local opts="$@"
 	$TESTDIR/../src/bin/lttng/$LTTNG_BIN track $opts >$OUTPUT_DEST
 	ret=$?
 	if [[ $expected_to_fail -eq "1" ]]; then
@@ -1185,8 +1483,9 @@ function lttng_track_fail()
 
 function lttng_untrack()
 {
-	local expected_to_fail=$1
-	local opts=$2
+	local expected_to_fail="$1"
+	shift 1
+	local opts="$@"
 	$TESTDIR/../src/bin/lttng/$LTTNG_BIN untrack $opts >$OUTPUT_DEST
 	ret=$?
 	if [[ $expected_to_fail -eq "1" ]]; then
@@ -1207,6 +1506,66 @@ function lttng_untrack_fail()
 	lttng_untrack 1 "$@"
 }
 
+function lttng_track_pid_ok()
+{
+	PID=$1
+	"$TESTDIR/../src/bin/lttng/$LTTNG_BIN" track --kernel --pid=$PID 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	ok $? "Lttng track pid on the kernel domain"
+}
+
+function lttng_untrack_kernel_all_ok()
+{
+	"$TESTDIR/../src/bin/lttng/$LTTNG_BIN" untrack --kernel --pid --all 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	ok $? "Lttng untrack all pid on the kernel domain"
+}
+
+function lttng_track_ust_ok()
+{
+	lttng_track_ok -u "$@"
+}
+
+function lttng_track_ust_fail()
+{
+	lttng_track_fail -u "$@"
+}
+
+function lttng_track_kernel_ok()
+{
+	lttng_track_ok -k "$@"
+}
+
+function lttng_track_kernel_fail()
+{
+	lttng_track_fail -k "$@"
+}
+
+function lttng_untrack_ust_ok()
+{
+	lttng_untrack_ok -u "$@"
+}
+
+function lttng_untrack_ust_fail()
+{
+	lttng_untrack_fail -u "$@"
+}
+
+function lttng_untrack_kernel_ok()
+{
+	lttng_untrack_ok -k "$@"
+}
+
+function lttng_untrack_kernel_fail()
+{
+	lttng_untrack_fail -k "$@"
+}
+
+function lttng_add_context_list()
+{
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN add-context --list 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	ret=$?
+	ok $ret "Context listing"
+}
+
 function add_context_lttng()
 {
 	local expected_to_fail="$1"
@@ -1245,6 +1604,55 @@ function add_context_kernel_fail()
 	add_context_lttng 1 -k "$@"
 }
 
+function wait_live_trace_ready ()
+{
+	local url=$1
+	local zero_client_match=0
+
+	diag "Waiting for live trace at url: $url"
+	while [ $zero_client_match -eq 0 ]; do
+		zero_client_match=$($BABELTRACE_BIN -i lttng-live $url | grep "0 client(s) connected" | wc -l)
+		sleep 0.5
+	done
+	pass "Waiting for live trace at url: $url"
+}
+
+function wait_live_viewer_connect ()
+{
+	local url=$1
+	local one_client_match=0
+
+	diag "Waiting for live viewers on url: $url"
+	while [ $one_client_match -eq 0 ]; do
+		one_client_match=$($BABELTRACE_BIN -i lttng-live $url | grep "1 client(s) connected" | wc -l)
+		sleep 0.5
+	done
+	pass "Waiting for live viewers on url: $url"
+}
+
+function validate_metadata_event ()
+{
+	local event_name=$1
+	local nr_event_id=$2
+	local trace_path=$3
+
+	local metadata_file=$(find $trace_path | grep metadata)
+	local metadata_path=$(dirname $metadata_file)
+
+	which $BABELTRACE_BIN >/dev/null
+	skip $? -ne 0 "Babeltrace binary not found. Skipping trace matches"
+
+	local count=$($BABELTRACE_BIN --output-format=ctf-metadata $metadata_path | grep $event_name | wc -l)
+
+	if [ "$count" -ne "$nr_event_id" ]; then
+		fail "Metadata match with the metadata of $count event(s) named $event_name"
+		diag "$count matching event id found in metadata"
+	else
+		pass "Metadata match with the metadata of $count event(s) named $event_name"
+	fi
+
+}
+
 function trace_matches ()
 {
 	local event_name=$1
@@ -1276,12 +1684,12 @@ function trace_match_only()
 	local count=$($BABELTRACE_BIN $trace_path | grep $event_name | wc -l)
 	local total=$($BABELTRACE_BIN $trace_path | wc -l)
 
-    if [ "$nr_iter" -eq "$count" ] && [ "$total" -eq "$nr_iter" ]; then
-        pass "Trace match with $total event $event_name"
-    else
-        fail "Trace match"
-        diag "$total event(s) found, expecting $nr_iter of event $event_name and only found $count"
-    fi
+	if [ "$nr_iter" -eq "$count" ] && [ "$total" -eq "$nr_iter" ]; then
+		pass "Trace match with $total event $event_name"
+	else
+		fail "Trace match"
+		diag "$total event(s) found, expecting $nr_iter of event $event_name and only found $count"
+	fi
 }
 
 function validate_trace
@@ -1339,6 +1747,36 @@ function validate_trace_count
 	ok $? "Read a total of $cnt events, expected $expected_count"
 }
 
+function validate_trace_count_range_incl_min_excl_max
+{
+	local event_name=$1
+	local trace_path=$2
+	local expected_min=$3
+	local expected_max=$4
+
+	which $BABELTRACE_BIN >/dev/null
+	if [ $? -ne 0 ]; then
+	    skip 0 "Babeltrace binary not found. Skipping trace validation"
+	fi
+
+	cnt=0
+	OLDIFS=$IFS
+	IFS=","
+	for i in $event_name; do
+		traced=$($BABELTRACE_BIN $trace_path 2>/dev/null | grep $i | wc -l)
+		if [ "$traced" -ge $expected_min ]; then
+			pass "Validate trace for event $i, $traced events"
+		else
+			fail "Validate trace for event $i"
+			diag "Found $traced occurences of $i"
+		fi
+		cnt=$(($cnt + $traced))
+	done
+	IFS=$OLDIFS
+	test $cnt -lt $expected_max
+	ok $? "Read a total of $cnt events, expected between [$expected_min, $expected_max["
+}
+
 function trace_first_line
 {
 	local trace_path=$1
@@ -1359,7 +1797,7 @@ function validate_trace_exp()
 	which $BABELTRACE_BIN >/dev/null
 	skip $? -ne 0 "Babeltrace binary not found. Skipping trace validation"
 
-	traced=$($BABELTRACE_BIN $trace_path 2>/dev/null | grep ${event_exp} | wc -l)
+	traced=$($BABELTRACE_BIN $trace_path 2>/dev/null | grep --extended-regexp ${event_exp} | wc -l)
 	if [ "$traced" -ne 0 ]; then
 		pass "Validate trace for expression '${event_exp}', $traced events"
 	else
@@ -1378,11 +1816,11 @@ function validate_trace_only_exp()
 	which $BABELTRACE_BIN >/dev/null
 	skip $? -ne 0 "Babeltrace binary not found. Skipping trace matches"
 
-	local count=$($BABELTRACE_BIN $trace_path | grep ${event_exp} | wc -l)
+	local count=$($BABELTRACE_BIN $trace_path | grep --extended-regexp ${event_exp} | wc -l)
 	local total=$($BABELTRACE_BIN $trace_path | wc -l)
 
 	if [ "$count" -ne 0 ] && [ "$total" -eq "$count" ]; then
-		pass "Trace match with $total for expression '${event_exp}"
+		pass "Trace match with $total for expression '${event_exp}'"
 	else
 		fail "Trace match"
 		diag "$total syscall event(s) found, only syscalls matching expression '${event_exp}' ($count occurrences) are expected"
@@ -1418,6 +1856,33 @@ function validate_trace_empty()
 	return $ret
 }
 
+function validate_directory_empty ()
+{
+	local trace_path="$1"
+
+	# Do not double quote `$trace_path` below as we want wildcards to be
+	# expanded.
+	files="$(ls -A $trace_path)"
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		fail "Failed to list content of directory \"$trace_path\""
+		return $ret
+	fi
+
+	nb_files="$(echo -n "$files" | wc -l)"
+	ok $nb_files "Directory \"$trace_path\" is empty"
+}
+
+function validate_trace_session_ust_empty()
+{
+	validate_directory_empty "$1"/ust
+}
+
+function validate_trace_session_kernel_empty()
+{
+	validate_trace_empty "$1"/kernel
+}
+
 function regenerate_metadata ()
 {
 	local expected_to_fail=$1
@@ -1454,7 +1919,7 @@ function regenerate_statedump ()
 		test "$ret" -ne "0"
 		ok $? "Expected fail on regenerate statedump $sess_name"
 	else
-		ok $ret "Metadata regenerate $sess_name"
+		ok $ret "Statedump regenerate $sess_name"
 	fi
 }
 
@@ -1468,6 +1933,31 @@ function regenerate_statedump_fail ()
 	regenerate_statedump 1 "$@"
 }
 
+function rotate_session ()
+{
+	local expected_to_fail=$1
+	local sess_name=$2
+
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN rotate $sess_name 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	ret=$?
+	if [[ $expected_to_fail -eq "1" ]]; then
+		test "$ret" -ne "0"
+		ok $? "Expected fail on rotate session $sess_name"
+	else
+		ok $ret "Rotate session $sess_name"
+	fi
+}
+
+function rotate_session_ok ()
+{
+	rotate_session 0 "$@"
+}
+
+function rotate_session_fail ()
+{
+	rotate_session 1 "$@"
+}
+
 function destructive_tests_enabled ()
 {
 	if [ ${LTTNG_ENABLE_DESTRUCTIVE_TESTS} = "will-break-my-system" ]; then
@@ -1476,3 +1966,86 @@ function destructive_tests_enabled ()
 		return 1
 	fi
 }
+
+function lttng_enable_rotation_timer ()
+{
+	local expected_to_fail=$1
+	local sess_name=$2
+	local period=$3
+
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN enable-rotation -s $sess_name --timer $period 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	ret=$?
+	if [[ $expected_to_fail -eq "1" ]]; then
+		test "$ret" -ne "0"
+		ok $? "Expected fail when setting periodic rotation ($period) of session $sess_name"
+	else
+		ok $ret "Set periodic rotation ($period) of session $sess_name"
+	fi
+}
+
+function lttng_enable_rotation_timer_ok ()
+{
+	lttng_enable_rotation_timer 0 $@
+}
+
+function lttng_enable_rotation_timer_fail ()
+{
+	lttng_enable_rotation_timer 1 $@
+}
+
+function lttng_enable_rotation_size ()
+{
+	local expected_to_fail=$1
+	local sess_name=$2
+	local size=$3
+
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN enable-rotation -s $sess_name --size $size 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	ret=$?
+	if [[ $expected_to_fail -eq "1" ]]; then
+		test "$ret" -ne "0"
+		ok $? "Expected fail on rotate session $sess_name"
+	else
+		ok $ret "Rotate session $sess_name"
+	fi
+}
+
+function lttng_enable_rotation_size_ok ()
+{
+	lttng_enable_rotation_size 0 $@
+}
+
+function lttng_enable_rotation_size_fail ()
+{
+	lttng_enable_rotation_size 1 $@
+}
+
+function lttng_clear_session ()
+{
+	local expected_to_fail=$1
+	local sess_name=$2
+
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN clear $sess_name 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	ret=$?
+	if [[ $expected_to_fail -eq "1" ]]; then
+		test "$ret" -ne "0"
+		ok $? "Expected fail on clear session $sess_name"
+	else
+		ok $ret "Clear session $sess_name"
+	fi
+}
+
+function lttng_clear_session_ok ()
+{
+	lttng_clear_session 0 $@
+}
+
+function lttng_clear_session_fail ()
+{
+	lttng_clear_session 1 $@
+}
+
+function lttng_clear_all ()
+{
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN clear --all 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	ok $? "Clear all lttng sessions"
+}
